(function($) {

  var TDVerify = function(options) {
    var obj = this;

    this.init = function() {
      // Replaces the default options with the ones supplied by the user.
      obj.config = $.extend(true, {
        'license' : null,

        'email'    : 'email',
        'phone'    : 'phone',
        'fname'    : 'first_name',
        'lname'    : 'last_name',
        'address1' : 'address1',
        'address2' : 'address2',
        'city'     : 'city',
        'state'    : 'state',
        'zip'      : 'zip',
        'ip'       : 'ip',

        'settings' : {
          'timeout'       : 5,
          'valid_email'   : true,  // True or 'mailbox' enables validation of email mailbox.
                                   // 'syntaxdomain' checks syntax and domain only.
          'valid_phone'   : false, // Validate phone number.
          'valid_postal'  : false, // Validate postal address.
          'reverse_email' : false, // Find postal address based on input email.
          'demographics'  : false  // Find demographics based on input email or
                                   // postal address. Must specify demographics package #.
        }
      }, options);

      obj.query = { 'settings' : {} };

      // If it fails the configuration check then we can't pass along the object.
      return obj.isValidConfig() ? this : false;
    };

    this.set = function(config) {
      obj.query = $.extend(true, obj.query, config);

      return this;
    };

    // This is the main process that fires off the AJAX request.
    this.process = function(config) {
      var url = obj.getURL();
      this.processData(config, url);
      return this;
    };



    this.processData = function(config, url) {
      var config = config || {};

      var ajaxConfig = {
        url: url,
        cache: true,
        dataType: 'jsonp',
        success: config.onSuccess,
        error: config.onError
        
      };

      // Run that AJAX
      //
      $.ajax(ajaxConfig);
      return this;
    };


    // This grabs the parameters and builds the URL.
    this.getURL = function() {
      var params = obj.getParamsFromDOM();
      return this.buildURL(params);
    };

    this.buildURL = function(params){

      var parsedParams = this.getParams(params);
      var urlParams = [];

      // Join all the key/pairs together and turn arrays into '+' delimited strings
      $.each(parsedParams, function(k, v) {
        if (v instanceof Array) {
          return urlParams.push(k + '=' + v.join('+'));
        }

        return urlParams.push(k + '=' + encodeURIComponent(v));
      });

      // Join all the parameters together for the URL
      this.url = 'http://api10.towerdata.com/person?' + urlParams.join('&');

      return this.url;

    };

    this.translate = function(key, category) {
      if (typeof key !== 'string') {
        return key;
      }

      var key = key.toLowerCase();

      var dict = {
        ''    : false,
        'off' : false,
        'on'  : true,

        'mailbox'      : 'email',
        'syntaxdomain' : 'email-domain-only',

        'valid_email'  : 'email',
        'valid_postal' : 'postal',
        'valid_phone'  : 'phone'
      };

      if (dict[category]) {
        return dict[category][key];
      }
      else if (dict[key]) {
        return dict[key];
      }
      else {
        return (dict[key] === false ? dict[key] : key);
      }
    };

    // This grabs a configuration value, turns it into a jQuery object if need be and cleans it up a bit..
    this.getValue = function(key, setting) {
      if (obj.query.settings[key] || obj.query[key]) {
        var value = (setting === true ? obj.query.settings[key] : obj.config[key]);
        return $.trim(value);
      }

      var id = (setting === true ? obj.config.settings[key] : obj.config[key]);
      var el;

      if (id instanceof jQuery) {
        el = id;
      }
      //else if (id instanceof Element) {
      else if (id && id.ownerDocument == document) {
        el = $(id);
      }
      else if (typeof id === 'string') {
        el = $('#' + id);
      }

      // Special case for checkboxes
      if (el && el.attr('type') === 'checkbox') {
        return el[0].checked;
      }

      return (el ? $.trim(el.val()) : false);
    };

    this.getParamsFromDOM = function(){

      var params = { 'validate' : [] };

      /* This will loop through all of the form elements provided
         and parse them out for their values, be them user-set, ids, or
         jQuery selectors.
      */

      for (var key in obj.config.settings) {
        var value = obj.translate(obj.query.settings[key] === undefined
                                    ? obj.getValue(key, true)
                                    : obj.query.settings[key]);
                                    

        // This inserts the data into the query object.
        obj.query.settings[key] = value;

        if (value === true) {
          if (key !== 'reverse_email' && key !== 'demographics') {
            var def = obj.translate(key);
            if (def) {
              params.validate.push(def);
            }
          }
        }
        else {
          if (key === 'valid_email') {
            params.validate.push(value);
          }
        }
      }
      return params;
    };



    // This will build all of the parameters for the URL primarily by getURL()
    this.getParams = function(params) {

      /********************************************************/

      // This fills up the forms with the proper data
      for (var key in obj.config) {
        if (key === 'settings') {
          continue;
        }

        if (obj.query[key]) {
          params[key] = obj.query[key];
          continue;
        }

        var value = obj.getValue(key);

        obj.query[key] = value;

        if (key === 'license') {
          value = obj.config[key];
        }

        if (value && value.length > 0) {
          params[key] = value;
        }
      } 

      /***********************************************************/

      // Validation sorts of logic
      if (params.email !== false) {
        if ($.inArray('email', params.validate) || $.inArray('syntax-domain-only', params.validate)) {
	  // Enable email corrections if processing an email
          params.correct = 'email';
        }
        if (obj.query.settings.reverse_email === true) {
          params.find = 'postal';
        }
      }

      if (obj.query.settings.demographics !== false) {
        params.demos = obj.query.settings.demographics;
      }
      else {
        if (obj.query.settings.valid_postal === false) {
          $.each(['address1', 'address2', 'city', 'state', 'zip'], function(i, el) {
            delete params[el];
          });
        }
      }

      if (params.validate.length < 1) {
        params.validate = 'none';
      }

      if (obj.query.settings.timeout > 0) {
        params.timeout = obj.query.settings.timeout;
      }

      /************************************************************/

      return params;
    };

    // Checks the configuration data and makes sure nothing is out of place.
    this.isValidConfig = function() {
      // There was a function here. It's gone now.

      return true;
    };

  };

  /* This is what is called by the user and is where the options are passed.
     It will also initialize everything and return a new copy of the TDVerify object.
  */
  $.tdverify = function(options) {
    var plugin = new TDVerify(options);

    // If init() fails then we can't continue.
    return plugin.init() ? plugin : undefined;
  };

})(jQuery);
